@@ -7,16 +7,16 @@ import asyncio
 import logging
 from datetime import datetime
 from json import dumps
-from typing import Any, Dict, List, Optional, Set
+from typing import Any, Dict, Optional, Set, cast
 
 from mirai import exceptions
 from mirai.api_provider import ApiProvider, Method
-from mirai.bus import EventBus
+from mirai.bus import AbstractEventBus
 
 logger = logging.getLogger(__name__)
 
 
-def _json_default(obj): # 支持 datetime
+def _json_default(obj):  # 支持 datetime
     if isinstance(obj, datetime):
         return int(obj.timestamp())
 
@@ -73,9 +73,9 @@ class Adapter(ApiProvider, AdapterInterface):
     """是否开启 single_mode，开启后与 session 将无效。"""
     session: str
     """从 mirai-api-http 处获得的 session。"""
-    buses: Set[EventBus]
+    buses: Set[AbstractEventBus]
     """注册的事件总线集合。"""
-    background: asyncio.Task
+    background: Optional[asyncio.Task]
     """背景事件循环任务。"""
     def __init__(self, verify_key: Optional[str], single_mode: bool = False):
         """
@@ -85,6 +85,7 @@ class Adapter(ApiProvider, AdapterInterface):
         self.single_mode = single_mode
         self.session = ''
         self.buses = set()
+        self.background = None
 
     @property
     def adapter_info(self):
@@ -105,20 +106,20 @@ class Adapter(ApiProvider, AdapterInterface):
                 for key in ['single_mode'] if info.get(key) is not None
             }
         )
-        adapter.session = info.get('session')
+        adapter.session = cast(str, info.get('session'))
         return adapter
 
-    def register_event_bus(self, *buses: List[EventBus]):
+    def register_event_bus(self, *buses: AbstractEventBus):
         """注册事件总线。
 
-        `*buses: List[EventBus]` 一个或多个事件总线。
+        `*buses: List[AbstractEventBus]` 一个或多个事件总线。
         """
         self.buses |= set(buses)
 
-    def unregister_event_bus(self, *buses: List[EventBus]):
+    def unregister_event_bus(self, *buses: AbstractEventBus):
         """解除注册事件总线。
 
-        `*buses: List[EventBus]` 一个或多个事件总线。
+        `*buses: List[AbstractEventBus]` 一个或多个事件总线。
         """
         self.buses -= set(buses)
 
@@ -127,7 +128,7 @@ class Adapter(ApiProvider, AdapterInterface):
         """登录到 mirai-api-http。"""
 
     @abc.abstractmethod
-    async def logout(self):
+    async def logout(self, terminate: bool = True):
         """登出。"""
 
     @abc.abstractmethod
@@ -158,3 +159,8 @@ class Adapter(ApiProvider, AdapterInterface):
         """停止背景事件循环。"""
         if self.background:
             self.background.cancel()
+
+    async def emit(self, event: str, *args, **kwargs):
+        """向事件总线发送一个事件。"""
+        coros = [bus.emit(event, *args, **kwargs) for bus in self.buses]
+        await asyncio.gather(*coros)
