@@ -5,12 +5,14 @@
 import asyncio
 import logging
 import random
-from typing import Optional
+from typing import Optional, cast
 
 import httpx
 
 from mirai import exceptions
-from mirai.adapters.base import Adapter, AdapterInterface, error_handler_async, json_dumps
+from mirai.adapters.base import (
+    Adapter, AdapterInterface, error_handler_async, json_dumps
+)
 from mirai.api_provider import Method
 from mirai.tasks import Tasks
 
@@ -22,7 +24,7 @@ def _parse_response(response: httpx.Response) -> dict:
     response.raise_for_status()
     result = response.json()
     if result.get('code', 0) != 0:
-        raise exceptions.ApiError(result['code'])
+        raise exceptions.ApiError(result)
     return result
 
 
@@ -81,7 +83,7 @@ class HTTPAdapter(Adapter):
         self.poll_interval = poll_interval
 
         self.qq = 0
-        self.headers = httpx.Headers() # 使用 headers 传递 session
+        self.headers = httpx.Headers()  # 使用 headers 传递 session
         self._tasks = Tasks()
 
     @property
@@ -103,41 +105,62 @@ class HTTPAdapter(Adapter):
             **{
                 key: info[key]
                 for key in ['host', 'port', 'poll_interval', 'single_mode']
-                if info.get(key) is not None
+                if key in info
             }
         )
-        adapter.session = info.get('session')
+        adapter.session = cast(str, info.get('session'))
         return adapter
 
     @_error_handler_async_local
-    async def _post(
-        self, client: httpx.AsyncClient, url: str, json: dict
-    ) -> dict:
+    async def _post(self, client: httpx.AsyncClient, url: str,
+                    json: dict) -> Optional[dict]:
         """调用 POST 方法。"""
         # 使用自定义的 json.dumps
         content = json_dumps(json).encode('utf-8')
-        response = await client.post(
-            url, content=content, headers={'Content-Type': 'application/json'}
-        )
-        logger.debug(f'[HTTP] 发送 POST 请求，地址{url}，状态 {response.status_code}。')
+        try:
+            response = await client.post(
+                url,
+                content=content,
+                headers={'Content-Type': 'application/json'},
+                timeout=10.
+            )
+            logger.debug(
+                f'[HTTP] 发送 POST 请求，地址{url}，状态 {response.status_code}。'
+            )
+        except httpx.TimeoutException:
+            logger.error(f'[HTTP] POST 请求超时，地址{url}。')
+            return None
         return _parse_response(response)
 
     @_error_handler_async_local
-    async def _get(
-        self, client: httpx.AsyncClient, url: str, params: dict
-    ) -> dict:
+    async def _get(self, client: httpx.AsyncClient, url: str,
+                   params: dict) -> Optional[dict]:
         """调用 GET 方法。"""
-        response = await client.get(url, params=params)
-        logger.debug(f'[HTTP] 发送 GET 请求，地址{url}，状态 {response.status_code}。')
+        try:
+            response = await client.get(url, params=params, timeout=10.)
+            logger.debug(
+                f'[HTTP] 发送 GET 请求，地址{url}，状态 {response.status_code}。'
+            )
+        except httpx.TimeoutException:
+            logger.error(f'[HTTP] GET 请求超时，地址{url}。')
+            return None
         return _parse_response(response)
 
     @_error_handler_async_local
     async def _post_multipart(
         self, client: httpx.AsyncClient, url: str, data: dict, files: dict
-    ) -> dict:
+    ) -> Optional[dict]:
         """调用 POST 方法，发送 multipart 数据。"""
-        response = await client.post(url, data=data, files=files)
-        logger.debug(f'[HTTP] 发送 POST 请求，地址{url}，状态 {response.status_code}。')
+        try:
+            response = await client.post(
+                url, data=data, files=files, timeout=30.
+            )
+            logger.debug(
+                f'[HTTP] 发送 POST 请求，地址{url}，状态 {response.status_code}。'
+            )
+        except httpx.TimeoutException:
+            logger.error(f'[HTTP] POST 请求超时，地址{url}。')
+            return None
         return _parse_response(response)
 
     @_error_handler_async_local
@@ -170,18 +193,19 @@ class HTTPAdapter(Adapter):
             logger.info(f'[HTTP] 成功登录到账号{qq}。')
 
     @_error_handler_async_local
-    async def logout(self):
-        if self.session:
-            async with httpx.AsyncClient(
-                base_url=self.host_name, headers=self.headers
-            ) as client:
-                await self._post(
-                    client, '/release', {
-                        "sessionKey": self.session,
-                        "qq": self.qq,
-                    }
-                )
-                logger.info(f"[HTTP] 从账号{self.qq}退出。")
+    async def logout(self, terminate: bool = True):
+        if self.session and not self.single_mode:
+            if terminate:
+                async with httpx.AsyncClient(
+                    base_url=self.host_name, headers=self.headers
+                ) as client:
+                    await self._post(
+                        client, '/release', {
+                            "sessionKey": self.session,
+                            "qq": self.qq,
+                        }
+                    )
+        logger.info(f"[HTTP] 从账号{self.qq}退出。")
 
     async def poll_event(self):
         """进行一次轮询，获取并处理事件。"""
@@ -195,13 +219,13 @@ class HTTPAdapter(Adapter):
                     self._get(client, '/fetchMessage', {'count': msg_count})
                 )['data']
 
-                coros = []
-                for bus in self.buses:
-                    for msg in msg_list:
-                        coros.append(bus.emit(msg['type'], msg))
+                coros = [self.emit(msg['type'], msg) for msg in msg_list]
                 await asyncio.gather(*coros)
 
-    async def call_api(self, api: str, method: Method = Method.GET, **params):
+    async def call_api(self,
+                       api: str,
+                       method: Method = Method.GET,
+                       **params) -> Optional[dict]:
         async with httpx.AsyncClient(
             base_url=self.host_name, headers=self.headers
         ) as client:
@@ -213,6 +237,8 @@ class HTTPAdapter(Adapter):
                 return await self._post_multipart(
                     client, f'/{api}', params['data'], params['files']
                 )
+            else:
+                return None
 
     async def _background(self):
         """开始轮询。"""
