@@ -3,7 +3,7 @@
 此模块提供 HTTP 回调适配器，适用于 mirai-api-http 的 webhook adapter。
 """
 import logging
-from typing import Optional
+from typing import Optional, cast
 
 from starlette.requests import Request
 from starlette.responses import JSONResponse
@@ -59,16 +59,19 @@ class WebHookAdapter(Adapter):
 
         async def endpoint(request: Request):
             # 鉴权（QQ 号和额外请求头）
-            if request.headers.get('bot') != self.session: # 验证 QQ 号
+            print(request.headers)
+            if request.headers.get('bot') != self.session:  # 验证 QQ 号
                 logger.debug(f"收到来自其他账号（{request.headers.get('bot')}）的事件。")
                 return
-            for key in self.extra_headers: # 验证请求头
-                key = key.lower() # HTTP headers 不区分大小写
-                if request.headers.get(key).lower(
-                ) != self.extra_headers[key].lower():
-                    logger.debug(
-                        f"请求头验证失败：expect {self.extra_headers[key].lower()}, " +
-                        f"got {request.headers.get(key).lower()}。"
+            for key in self.extra_headers:  # 验证请求头
+                key = key.lower()  # HTTP headers 不区分大小写
+                request_value = request.headers.get(key, '').lower()
+                expect_value = self.extra_headers[key].lower()
+                if (request_value != expect_value
+                    ) and (request_value != '[' + expect_value + ']'):
+                    logger.info(
+                        f"请求头验证失败：expect [{expect_value}], " +
+                        f"got {request_value}。"
                     )
                     return JSONResponse(
                         status_code=401, content={'error': 'Unauthorized'}
@@ -106,10 +109,10 @@ class WebHookAdapter(Adapter):
                 for key in [
                     'route', 'extra_headers', 'enable_quick_response',
                     'single_mode'
-                ] if info.get(key) is not None
+                ] if key in info
             }
         )
-        adapter.session = info.get('session')
+        adapter.session = cast(str, info.get('session'))
         return adapter
 
     async def login(self, qq: int):
@@ -117,7 +120,7 @@ class WebHookAdapter(Adapter):
         self.session = str(qq)
         logger.info(f'[WebHook] 成功登录到账号{qq}。')
 
-    async def logout(self):
+    async def logout(self, terminate: bool = True):
         """WebHook 不需要登出。直接返回。"""
         logger.info(f"[WebHook] 从账号{self.session}退出。")
 
@@ -143,8 +146,7 @@ class WebHookAdapter(Adapter):
 
     async def handle_event(self, event):
         try:
-            for bus in self.buses:
-                await bus.emit(event['type'], event)
+            await self.emit(event['type'], event)
         except WebHookAdapter.QuickResponse as response:
             # 快速响应，直接返回。
             return response.data
