@@ -2,24 +2,30 @@
 """
 此模块提供正向 WebSocket 适配器，适用于 mirai-api-http 的 websocket adapter。
 """
+
 import asyncio
 import json
 import logging
 import random
 from collections import defaultdict, deque
-from typing import Optional
+from typing import Dict, Optional, cast
 
-import websockets
+from websockets.client import WebSocketClientProtocol, connect
+from websockets.exceptions import (
+    ConnectionClosed, ConnectionClosedOK, InvalidURI
+)
 
 from mirai import exceptions
-from mirai.adapters.base import Adapter, AdapterInterface, error_handler_async, json_dumps
+from mirai.adapters.base import (
+    Adapter, AdapterInterface, error_handler_async, json_dumps
+)
 from mirai.api_provider import Method
 from mirai.tasks import Tasks
 
 logger = logging.getLogger(__name__)
 
 _error_handler_async_local = error_handler_async(
-    (ConnectionRefusedError, websockets.InvalidURI)
+    (ConnectionRefusedError, InvalidURI)
 )
 
 
@@ -32,7 +38,7 @@ class WebSocketAdapter(Adapter):
     """mirai-api-http 配置的同步 ID。"""
     qq: int
     """机器人的 QQ 号。"""
-    connection: websockets.WebSocketClientProtocol
+    connection: WebSocketClientProtocol
     """WebSocket 客户端连接。"""
     def __init__(
         self,
@@ -70,13 +76,13 @@ class WebSocketAdapter(Adapter):
 
         self.host_name = f'{host}:{port}/all'
 
-        self.sync_id = sync_id # 这个神奇的 sync_id，默认值 -1，居然是个字符串
+        self.sync_id = sync_id  # 这个神奇的 sync_id，默认值 -1，居然是个字符串
         # 既然这样不如把 sync_id 全改成字符串好了
 
         # 接收 WebSocket 数据的 Task
-        self._receiver_task = None
+        self._receiver_task: Optional[asyncio.Task] = None
         # 用于临时保存接收到的数据，以便根据 sync_id 进行同步识别
-        self._recv_dict = defaultdict(deque)
+        self._recv_dict: Dict[str, deque] = defaultdict(deque)
         # 本地同步 ID，每次调用 API 递增。
         self._local_sync_id = random.randint(1, 1024) * 1024
         # 事件处理任务管理器
@@ -101,10 +107,10 @@ class WebSocketAdapter(Adapter):
             **{
                 key: info[key]
                 for key in ['host', 'port', 'sync_id', 'single_mode']
-                if info.get(key) is not None
+                if key in info
             }
         )
-        adapter.session = info.get('session')
+        adapter.session = cast(str, info.get('session'))
         return adapter
 
     @_error_handler_async_local
@@ -127,7 +133,7 @@ class WebSocketAdapter(Adapter):
                 data = response['data']
 
                 if data.get('code', 0) != 0:
-                    raise exceptions.ApiError(data['code'])
+                    raise exceptions.ApiError(data)
 
                 logger.debug(
                     f"[WebSocket] 收到 WebSocket 数据，同步 ID：{response['syncId']}。"
@@ -135,9 +141,9 @@ class WebSocketAdapter(Adapter):
                 self._recv_dict[response['syncId']].append(data)
             except KeyError:
                 logger.error(f'[WebSocket] 不正确的数据：{response}')
-            except websockets.ConnectionClosedOK:
+            except ConnectionClosedOK:
                 return
-            except websockets.ConnectionClosed as e:
+            except ConnectionClosed as e:
                 logger.error(
                     f'[WebSocket] WebSocket 通道意外关闭。code: {e.code}, reason: {e.reason}'
                 )
@@ -145,14 +151,14 @@ class WebSocketAdapter(Adapter):
 
     async def _recv(self, sync_id: str = '-1') -> dict:
         """接收并解析 websocket 数据。"""
-        for _ in range(100):
+        for _ in range(600):
             if self._recv_dict[sync_id]:
                 return self._recv_dict[sync_id].popleft()
             else:
                 # 如果没有对应同步 ID 的数据，则等待 websocket 数据
                 # 目前存在问题：如果 mah 发回的数据不含 sync_id，
                 # 这里就会无限循环……
-                # 所以还是限制 100 次好了。
+                # 所以还是限制次数好了。
                 await asyncio.sleep(0.1)
         else:
             raise TimeoutError(
@@ -163,25 +169,23 @@ class WebSocketAdapter(Adapter):
     async def login(self, qq: int):
         headers = {
             'verifyKey': self.verify_key or
-                         '', # 关闭认证时，WebSocket 可传入任意 verify_key
-            'qq': qq,
+                         '',  # 关闭认证时，WebSocket 可传入任意 verify_key
+            'qq': str(qq),
         }
         if self.session:
             headers['sessionKey'] = self.session
 
-        self.connection = await websockets.connect(
-            self.host_name, extra_headers=headers
-        )
+        self.connection = await connect(self.host_name, extra_headers=headers)
         self._receiver_task = asyncio.create_task(self._receiver())
 
-        verify_response = await self._recv('') # 神奇现象：这里的 syncId 是个空字符串
+        verify_response = await self._recv('')  # 神奇现象：这里的 syncId 是个空字符串
         self.session = verify_response['session']
 
         self.qq = qq
         logger.info(f'[WebSocket] 成功登录到账号{qq}。')
 
     @_error_handler_async_local
-    async def logout(self):
+    async def logout(self, terminate: bool = True):
         if self.connection:
             await self.connection.close()
 
@@ -193,11 +197,10 @@ class WebSocketAdapter(Adapter):
         """获取并处理事件。"""
         event = await self._recv(self.sync_id)
 
-        for bus in self.buses:
-            self._tasks.create_task(bus.emit(event['type'], event))
+        self._tasks.create_task(self.emit(event['type'], event))
 
     async def call_api(self, api: str, method: Method = Method.GET, **params):
-        self._local_sync_id += 1 # 使用不同的 sync_id
+        self._local_sync_id += 1  # 使用不同的 sync_id
         sync_id = str(self._local_sync_id)
         content = {
             'syncId': sync_id,
@@ -215,7 +218,10 @@ class WebSocketAdapter(Adapter):
 
         await self.connection.send(json_dumps(content))
         logger.debug(f"[WebSocket] 发送 WebSocket 数据，同步 ID：{sync_id}。")
-        return await self._recv(sync_id)
+        try:
+            return await self._recv(sync_id)
+        except TimeoutError as e:
+            logger.debug(e)
 
     async def _background(self):
         """开始接收事件。"""
