@@ -2,31 +2,41 @@
 """
 此模块提供 API 调用与返回数据解析相关。
 """
+import abc
+import logging
 from enum import Enum, Flag
 from pathlib import Path
-from typing import List, Optional, Type, Union
+from typing import (
+    TYPE_CHECKING, Any, Generic, List, Optional, Type, TypeVar, Union, cast
+)
 
 import aiofiles
 
-try:
+if TYPE_CHECKING:
     from typing import Literal
-except ImportError:
-    from typing_extensions import Literal
+else:
+    try:
+        from typing import Literal
+    except ImportError:
+        from typing_extensions import Literal
+
+from pydantic import validator
 
 from mirai.api_provider import ApiProvider, Method
 from mirai.models.base import (
     MiraiBaseModel, MiraiIndexedMetaclass, MiraiIndexedModel
 )
 from mirai.models.entities import (
-    Friend, Group, GroupConfig, GroupMember, MemberInfo
+    Friend, Group, GroupConfigModel, GroupMember, MemberInfoModel
 )
 from mirai.models.events import (
     FriendMessage, GroupMessage, OtherClientMessage, RequestEvent,
     StrangerMessage, TempMessage
 )
-from mirai.models.message import Image, MessageChain, Voice
+from mirai.models.message import Image, MessageChain, MessageComponent, Voice
 from mirai.utils import async_
-from pydantic import Field, validator
+
+logger = logging.getLogger(__name__)
 
 
 class Response(MiraiBaseModel):
@@ -35,6 +45,16 @@ class Response(MiraiBaseModel):
     """状态码。"""
     msg: str
     """消息。"""
+    data: Optional[Any] = None
+    """返回数据。"""
+    def __getattr__(self, item):
+        return getattr(self.data, item)
+
+    def __iter__(self):
+        return iter(self.data)
+
+    def __getitem__(self, item):
+        return self.data[item]
 
 
 class AboutResponse(Response):
@@ -42,6 +62,28 @@ class AboutResponse(Response):
     data: dict
 
 
+class SessionInfoResponse(Response):
+    """机器人信息。"""
+    data: 'SessionInfoResponse.Data'
+
+    class Data(MiraiBaseModel):
+        session_key: str = 'SINGLE_SESSION'
+        """session_key，不知道干啥用的。"""
+        qq: Friend
+        """机器人信息。"""
+
+    @property
+    def qq(self):
+        return self.data.qq.id
+
+    @property
+    def nickname(self):
+        return self.data.qq.nickname
+
+
+SessionInfoResponse.update_forward_refs()
+
+
 class MessageFromIdResponse(Response):
     '''通过 message_id 获取的消息。'''
     data: Union[FriendMessage, GroupMessage, TempMessage, StrangerMessage,
@@ -79,6 +121,9 @@ class Sex(str, Enum):
     Male = 'MALE'
     Female = 'FEMALE'
 
+    def __repr__(self) -> str:
+        return repr(self.value)
+
 
 class ProfileResponse(MiraiBaseModel):
     """好友资料。"""
@@ -101,6 +146,15 @@ class MessageResponse(Response):
     message_id: int
     """消息的 message_id。"""
 
+class DownloadInfo(MiraiBaseModel):
+    """文件的下载信息。"""
+    sha1: str
+    """文件的 SHA1。"""
+    md5: str
+    """文件的 MD5。"""
+    url: str
+    """文件的下载地址。"""
+
 
 class File(MiraiBaseModel):
     """文件对象。"""
@@ -116,11 +170,13 @@ class File(MiraiBaseModel):
     """群信息或好友信息。"""
     is_file: bool
     """是否是文件。"""
-    is_dictionary: bool
+    is_directory: bool
     """是否是文件夹。"""
+    download_info: Optional[DownloadInfo] = None
+    """文件的下载信息。"""
 
 
-File.update_forward_refs() # 支持 model 引用自己的类型
+File.update_forward_refs()  # 支持 model 引用自己的类型
 
 
 class FileListResponse(Response):
@@ -150,22 +206,25 @@ class ApiMetaclass(MiraiIndexedMetaclass):
 
         if name == 'ApiModel':
             cls.__apimodel__ = new_cls
+            new_cls.__indexes__ = {}
             return new_cls
 
-        if not cls.__apimodel__: # ApiBaseModel 构造时，ApiModel 还未构造
+        if not cls.__apimodel__:  # ApiBaseModel 构造时，ApiModel 还未构造
             return new_cls
 
         for base in bases:
             if issubclass(base, cls.__apimodel__):
                 info = new_cls.Info
-                if hasattr(info, 'name'):
+                if hasattr(info, 'name') and info.name:
                     base.__indexes__[info.name] = new_cls
-                if hasattr(info, 'alias'):
+                if hasattr(info, 'alias') and info.alias:
                     base.__indexes__[info.alias] = new_cls
 
                 # 获取 API 参数名
-                if hasattr(new_cls, '__annotations__'):
-                    info.parameter_names = list(new_cls.__annotations__)
+                if hasattr(new_cls, '__fields__'):
+                    info.parameter_names = list(new_cls.__fields__)
+                else:
+                    info.parameter_names = []
                 break
 
         return new_cls
@@ -179,24 +238,28 @@ class ApiBaseModel(MiraiIndexedModel, metaclass=ApiMetaclass):
     """
 
 
+TModel = TypeVar('TModel', bound=MiraiBaseModel)
+
+
 class ApiModel(ApiBaseModel):
     """API 模型。"""
     class Info():
         """API 的信息。"""
         name = ""
         alias = ""
-        method = Method.GET
-        response_type = Response
+        response_type: Type[MiraiBaseModel] = Response
+        parameter_names: List[str] = []
 
     def __init__(self, *args, **kwargs):
         # 解析参数列表，将位置参数转化为具名参数
         parameter_names = self.Info.parameter_names
         if len(args) > len(parameter_names):
             raise TypeError(
-                f'`{self.Info.alias}`需要{len(parameter_names)}个参数，但传入了{len(args)}个。'
+                f'`{self.Info.alias}`需要{len(parameter_names)}个参数，' +
+                '但传入了{len(args)}个。'
             )
         for name, value in zip(parameter_names, args):
-            if kwargs.get(name):
+            if name in kwargs:
                 raise TypeError(
                     f'在 `{self.Info.alias}` 中，具名参数 `{name}` 与位置参数重复。'
                 )
@@ -208,15 +271,16 @@ class ApiModel(ApiBaseModel):
     @classmethod
     def get_subtype(cls, name: str) -> Type['ApiModel']:
         try:
-            return super().get_subtype(name)
+            return cast(Type['ApiModel'], super().get_subtype(name))
         except ValueError as e:
             raise ValueError(f'`{name}` 不是可用的 API！') from e
 
-    class Proxy():
+    class Proxy(Generic[TModel]):
         """API 代理类。由 API 构造，提供对适配器的访问。
 
         `Proxy` 提供更加简便的调用 API 的写法。`Proxy` 对象一般由 `Mirai.__getattr__` 获得，
-        这个方法依托于 `MiraiIndexedModel` 的子类获取机制，支持按照命名规范转写的 API，所有的 API 全部使用小写字母及下划线命名。
+        这个方法依托于 `MiraiIndexedModel` 的子类获取机制，支持按照命名规范转写的 API，
+        所有的 API 全部使用小写字母及下划线命名。
 
         对于 GET 方法的 API，可以使用 `get` 方法：
         ```py
@@ -242,7 +306,8 @@ class ApiModel(ApiBaseModel):
         ```
 
         `ApiProxy` 同时提供位置参数支持。比如上面的例子中，没有使用具名参数，而是使用位置参数，
-        这可以让 API 调用更简洁。参数的顺序可参照 mirai-api-http 的[文档](https://project-mirai.github.io/mirai-api-http/api/API.html)。
+        这可以让 API 调用更简洁。参数的顺序可参照 mirai-api-http
+        的[文档](https://project-mirai.github.io/mirai-api-http/api/API.html)。
         除去`sessionKey`由适配器自动指定外，其余参数可按顺序传入。具名参数仍然可用，适当地使用具名参数可增强代码的可读性。
         """
         def __init__(
@@ -254,10 +319,10 @@ class ApiModel(ApiBaseModel):
         async def _call_api(
             self,
             method: Method = Method.GET,
-            response_type: Optional[Type[MiraiBaseModel]] = None,
-            args: Optional[list] = None,
+            response_type: Optional[Type[TModel]] = None,
+            args: Optional[Union[list, tuple]] = None,
             kwargs: Optional[dict] = None
-        ) -> MiraiBaseModel:
+        ) -> Optional[TModel]:
             """调用 API。
 
             将结果解析为 Model。
@@ -267,41 +332,50 @@ class ApiModel(ApiBaseModel):
 
             api = self.api_type(*args, **kwargs)
             info = self.api_type.Info
-            if hasattr(api, 'call'):
+            logger.debug(f'调用 API：{repr(api)}')
+            if isinstance(api, CustomApiModel):
                 raw_response = await api.call(self.api_provider)
             else:
-                raw_response = await async_(
-                    self.api_provider.call_api(
-                        api=info.name,
-                        method=method,
-                        **api.dict(by_alias=True, exclude_none=True)
-                    )
+                raw_response = await self.api_provider.call_api(
+                    api=info.name,
+                    method=method,
+                    **api.dict(by_alias=True, exclude_none=True)
                 )
+
             # 如果 API 无法调用，raw_response 为空
             if not raw_response:
                 return None
             # 解析 API 返回数据
-            response_type = response_type or info.response_type
+            response_type = cast(
+                Type[TModel], response_type or info.response_type
+            )
             return response_type.parse_obj(raw_response)
 
-        async def get(self, *args, **kwargs) -> MiraiBaseModel:
+        async def get(self, *args, **kwargs) -> Optional[TModel]:
             """获取。对于 GET 方法的 API，调用此方法。"""
             return await self._call_api(
                 method=Method.GET, args=args, kwargs=kwargs
             )
 
-        async def set(self, *args, **kwargs) -> MiraiBaseModel:
+        async def set(self, *args, **kwargs) -> Optional[TModel]:
             """设置。对于 POST 方法的 API，可调用此方法。"""
             return await self._call_api(
                 method=Method.POST, args=args, kwargs=kwargs
             )
 
-        async def __call__(self, *args, **kwargs) -> MiraiBaseModel:
+        async def __call__(self, *args, **kwargs):
             return await self.get(*args, **kwargs)
 
 
+class CustomApiModel(ApiBaseModel):
+    """自定义调用方式的 API。"""
+    @abc.abstractmethod
+    async def call(self, api_provider: ApiProvider):
+        """调用 API。"""
+
+
 class ApiGet(ApiModel):
-    class Proxy(ApiModel.Proxy):
+    class Proxy(ApiModel.Proxy[TModel]):
         async def set(self, *args, **kwargs):
             """GET 方法的 API 不具有 `set`。
 
@@ -309,9 +383,12 @@ class ApiGet(ApiModel):
             """
             raise TypeError(f'`{self.command}` 不支持 `set` 方法。')
 
+        async def __call__(self, *args, **kwargs) -> Optional[TModel]:
+            return await self.get(*args, **kwargs)
+
 
 class ApiPost(ApiModel):
-    class Proxy(ApiModel.Proxy):
+    class Proxy(ApiModel.Proxy[TModel]):
         """POST 方法的 API 代理对象。"""
         async def get(self, *args, **kwargs):
             """POST 方法的 API 不具有 `get`。
@@ -320,7 +397,7 @@ class ApiPost(ApiModel):
             """
             raise TypeError(f'`{self.command}` 不支持 `get` 方法。')
 
-        async def __call__(self, *args, **kwargs):
+        async def __call__(self, *args, **kwargs) -> Optional[TModel]:
             return await self.set(*args, **kwargs)
 
 
@@ -333,7 +410,7 @@ class ApiRest(ApiModel):
         response_type = MiraiBaseModel
         response_type_post = Response
 
-    class Proxy(ApiModel.Proxy):
+    class Proxy(ApiModel.Proxy[TModel]):
         """RESTful 的 API 代理对象。
 
         直接调用时，传入 GET 和 POST 的公共参数，返回一个 `ApiRest.Proxy.Partial` 对象，
@@ -341,20 +418,21 @@ class ApiRest(ApiModel):
         """
         def __call__(self, *args, **kwargs) -> 'ApiRest.Proxy.Partial':
             return ApiRest.Proxy.Partial(
-                self.api_provider, self.api_type, args, kwargs
+                self.api_provider, cast(Type['ApiRest'], self.api_type), args,
+                kwargs
             )
 
-        class Partial(ApiModel.Proxy):
+        class Partial(ApiModel.Proxy[TModel]):
             """RESTful 的 API 代理对象（已传入公共参数）。"""
             def __init__(
                 self, api_provider: ApiProvider, api_type: Type['ApiRest'],
-                partial_args: list, partial_kwargs: dict
+                partial_args: Union[list, tuple], partial_kwargs: dict
             ):
                 super().__init__(api_provider=api_provider, api_type=api_type)
                 self.partial_args = partial_args
                 self.partial_kwargs = partial_kwargs
 
-            async def get(self, *args, **kwargs) -> MiraiBaseModel:
+            async def get(self, *args, **kwargs) -> Optional[TModel]:
                 """获取。"""
                 return await self._call_api(
                     method=Method.RESTGET,
@@ -365,11 +443,15 @@ class ApiRest(ApiModel):
                     }
                 )
 
-            async def set(self, *args, **kwargs) -> MiraiBaseModel:
+            async def set(self, *args, **kwargs) -> Optional[TModel]:
                 """设置。"""
                 return await self._call_api(
                     method=Method.RESTPOST,
-                    response_type=self.api_type.Info.response_type_post,
+                    response_type=cast(
+                        Type[TModel],
+                        cast(Type[ApiRest],
+                             self.api_type).Info.response_type_post
+                    ),
                     args=[*self.partial_args, *args],
                     kwargs={
                         **self.partial_kwargs,
@@ -386,9 +468,17 @@ class About(ApiGet):
         response_type = AboutResponse
 
 
+class SessionInfo(ApiGet):
+    """获取机器人信息。"""
+    class Info(ApiGet.Info):
+        name = "sessionInfo"
+        alias = "session_info"
+        response_type = SessionInfoResponse
+
+
 class MessageFromId(ApiGet):
     """通过 message_id 获取消息。"""
-    target: int
+    id: int
     """获取消息的 message_id。"""
     class Info(ApiGet.Info):
         name = "messageFromId"
@@ -454,9 +544,9 @@ class MemberProfile(ApiGet):
 
 class SendMessage(ApiBaseModel):
     """发送消息的 API 的方法复用，不作为 API 使用。"""
-    message_chain: Union[MessageChain, list]
+    # message_chain: Union[MessageChain, List[Union[MessageComponent, str]], str]
 
-    @validator('message_chain')
+    @validator('message_chain', check_fields=False)
     def _validate_message_chain(cls, value: Union[MessageChain, list]):
         if isinstance(value, list):
             return MessageChain.parse_obj(value)
@@ -468,7 +558,7 @@ class SendFriendMessage(ApiPost, SendMessage):
     """发送好友消息。"""
     target: int
     """发送消息目标好友的 QQ 号。"""
-    message_chain: Union[MessageChain, list]
+    message_chain: Union[MessageChain, List[Union[MessageComponent, str]], str]
     """消息链。"""
     quote: Optional[int] = None
     """可选。引用一条消息的 message_id 进行回复。"""
@@ -482,7 +572,7 @@ class SendGroupMessage(ApiPost, SendMessage):
     """发送群消息。"""
     target: int
     """发送消息目标群的群号。"""
-    message_chain: Union[MessageChain, list]
+    message_chain: Union[MessageChain, List[Union[MessageComponent, str]], str]
     """消息链。"""
     quote: Optional[int] = None
     """可选。引用一条消息的 message_id 进行回复。"""
@@ -498,7 +588,7 @@ class SendTempMessage(ApiPost, SendMessage):
     """临时会话对象 QQ 号。"""
     group: int
     """临时会话对象群号。"""
-    message_chain: Union[MessageChain, list]
+    message_chain: Union[MessageChain, List[Union[MessageComponent, str]], str]
     """消息链。"""
     quote: Optional[int] = None
     """可选。引用一条消息的 message_id 进行回复。"""
@@ -538,6 +628,8 @@ class FileList(ApiGet):
     """文件夹 id，空串为根目录。"""
     target: int
     """群号或好友 QQ 号。"""
+    with_download_info: bool = False
+    """是否携带下载信息。"""
     class Info(ApiGet.Info):
         name = "file/list"
         alias = "file_list"
@@ -550,6 +642,8 @@ class FileInfo(ApiGet):
     """文件 id。"""
     target: int
     """群号或好友 QQ 号。"""
+    with_download_info: bool = False
+    """是否携带下载信息。"""
     class Info(ApiGet.Info):
         name = "file/info"
         alias = "file_info"
@@ -562,7 +656,7 @@ class FileMkdir(ApiPost):
     """父目录 id。"""
     target: int
     """群号或好友 QQ 号。"""
-    dictionary_name: str
+    directory_name: str
     """新建文件夹名。"""
     class Info(ApiPost.Info):
         name = "file/mkdir"
@@ -610,7 +704,37 @@ class FileRename(ApiPost):
         response_type = Response
 
 
-class UploadImage(ApiGet):
+class FileUpload(ApiPost, CustomApiModel):
+    """文件上传。（暂时不可用）"""
+    type: Literal["group"]
+    """上传的文件类型。"""
+    target: int
+    """群号。"""
+    file: Union[str, Path]
+    """上传的文件的本地路径。"""
+    path: str = ''
+    """上传目录的 id，空串为上传到根目录。"""
+    async def call(self, api_provider: ApiProvider):
+        async with aiofiles.open(self.file, 'rb') as f:
+            file = await f.read()
+        return await api_provider.call_api(
+            'file/upload',
+            method=Method.MULTIPART,
+            data={
+                'type': self.type,
+                'target': self.target,
+                'path': self.path,
+            },
+            files={'file': file}
+        )
+
+    class Info(ApiPost.Info):
+        name = "file/upload"
+        alias = "file_upload"
+        response_type = File
+
+
+class UploadImage(ApiPost, CustomApiModel):
     """图片文件上传。"""
     type: Literal["friend", "group", "temp"]
     """上传的图片类型。"""
@@ -626,13 +750,13 @@ class UploadImage(ApiGet):
             files={'img': img}
         )
 
-    class Info(ApiGet.Info):
+    class Info(ApiPost.Info):
         name = "uploadImage"
         alias = "upload_image"
         response_type = Image
 
 
-class UploadVoice(ApiGet):
+class UploadVoice(ApiPost, CustomApiModel):
     """语音文件上传。"""
     type: Literal["group"]
     """上传的语音类型。"""
@@ -648,7 +772,7 @@ class UploadVoice(ApiGet):
             files={'voice': voice}
         )
 
-    class Info(ApiGet.Info):
+    class Info(ApiPost.Info):
         name = "uploadVoice"
         alias = "upload_voice"
         response_type = Voice
@@ -668,7 +792,7 @@ class Mute(ApiPost):
     """禁言群成员。"""
     target: int
     """指定群的群号。"""
-    memder_id: int
+    member_id: int
     """指定群成员的 QQ 号。"""
     time: int
     """禁言时间，单位为秒，最多30天，默认为0。"""
@@ -682,7 +806,7 @@ class Unmute(ApiPost):
     """解除群成员禁言。"""
     target: int
     """指定群的群号。"""
-    memder_id: int
+    member_id: int
     """指定群成员的 QQ 号。"""
     class Info(ApiPost.Info):
         name = "unmute"
@@ -694,7 +818,7 @@ class Kick(ApiPost):
     """移出群成员。"""
     target: int
     """指定群的群号。"""
-    memder_id: int
+    member_id: int
     """指定群成员的 QQ 号。"""
     msg: str = ""
     """可选。信息。"""
@@ -748,12 +872,12 @@ class GroupConfig(ApiRest):
     """获取或修改群设置。"""
     target: int
     """群号。"""
-    config: Optional[GroupConfig] = None
+    config: Optional[GroupConfigModel] = None
     """仅修改时可用。群设置。"""
     class Info(ApiRest.Info):
         name = "groupConfig"
         alias = "group_config"
-        response_type = GroupConfig
+        response_type = GroupConfigModel
         response_type_post = Response
 
 
@@ -763,12 +887,12 @@ class MemberInfo(ApiRest):
     """群号。"""
     member_id: int
     """指定群成员的 QQ 号。"""
-    info: Optional[MemberInfo] = None
+    info: Optional[MemberInfoModel] = None
     """仅修改时可用。群成员资料。"""
     class Info(ApiRest.Info):
         name = "memberInfo"
         alias = "member_info"
-        response_type = MemberInfo
+        response_type = MemberInfoModel
         response_type_post = Response
 
 
@@ -887,6 +1011,7 @@ class RespMemberJoinRequestEvent(ApiPost):
 
 
 class RespBotInvitedJoinGroupRequestEvent(ApiPost):
+    """响应被邀请入群申请。"""
     event_id: int
     """响应申请事件的标识。"""
     from_id: int
@@ -917,7 +1042,7 @@ class RespBotInvitedJoinGroupRequestEvent(ApiPost):
 
 class CmdExecute(ApiPost):
     """执行命令。"""
-    command: Union[MessageChain, list]
+    command: Union[MessageChain, List[Union[MessageComponent, str]], str]
     """命令。"""
     @validator('command')
     def _validate_command(cls, value):
