@@ -14,7 +14,7 @@ from pydantic import Field
 
 from mirai.models.base import MiraiIndexedModel
 from mirai.models.entities import (
-    Friend, Group, GroupMember, Permission, Sender
+    Entity, Friend, Group, GroupMember, Permission, Sender, Subject
 )
 from mirai.models.message import MessageChain
 
@@ -26,6 +26,13 @@ class Event(MiraiIndexedModel):
     """
     type: str
     """事件名。"""
+    def __repr__(self):
+        return self.__class__.__name__ + '(' + ', '.join(
+            (
+                f'{k}={repr(v)}'
+                for k, v in self.__dict__.items() if k != 'type' and v
+            )
+        ) + ')'
 
 
 ###############################
@@ -771,6 +778,22 @@ class BotInvitedJoinGroupRequestEvent(RequestEvent):
     """邀请消息。"""
 
 
+class NudgeEvent(Event):
+    """头像戳一戳事件。"""
+    type: str = "NudgeEvent"
+    """事件名。"""
+    from_id: int
+    """动作发出者的 QQ 号。"""
+    target: int
+    """动作目标的 QQ 号。"""
+    subject: Subject
+    """来源。"""
+    action: str
+    """戳一戳类型。"""
+    suffix: str
+    """自定义戳一戳内容。"""
+
+
 ###############################
 # Command Event
 
@@ -820,6 +843,8 @@ class MessageEvent(Event):
     """
     type: str
     """事件名。"""
+    sender: Entity
+    """发送消息的来源。"""
     message_chain: MessageChain
     """消息内容。"""
 
@@ -856,7 +881,6 @@ class GroupMessage(MessageEvent):
     """发送消息的群成员。"""
     message_chain: MessageChain
     """消息内容。"""
-
     @property
     def group(self) -> Group:
         return self.sender.group
@@ -877,6 +901,9 @@ class TempMessage(MessageEvent):
     """发送消息的群成员。"""
     message_chain: MessageChain
     """消息内容。"""
+    @property
+    def group(self) -> Group:
+        return self.sender.group
 
 
 class StrangerMessage(MessageEvent):
@@ -956,6 +983,7 @@ __all__ = [
     'MemberUnmuteEvent',
     'MessageEvent',
     'NewFriendRequestEvent',
+    'NudgeEvent',
     'OtherClientMessage',
     'RequestEvent',
     'StrangerMessage',
