@@ -1,7 +1,11 @@
+# -*- coding: utf-8 -*-
+"""
+此模块提供了管理多个异步任务的一种方式。
+"""
 import asyncio
 
 
-class Tasks():
+class Tasks:
     """管理多个异步任务的类。"""
     def __init__(self):
         self._tasks = set()
